%{
/*
  Sample Lex program

  To compile this program,

  % lex calc.l
  % gcc lex.yy.c -ll
*/

/* You can put any C code here. */

#define END		0	/* end of input */
#define BADTOKEN	1	/* invalid token */
#define NUMBER		2	/* decimal number */
#define HEXNUMBER	3	/* hex number */
#define RETURN		4
#define PLUS		5
#define MINUS		6

%}

digit				[0-9]
white_space			[ \t]
hexdigit			[0-9A-Fa-f]

%%

{digit}+			return NUMBER;
0[Xx]{hexdigit}+		return HEXNUMBER;

"+"				return PLUS;
"-"				return MINUS;

"\n"				return RETURN;

{white_space}+			;	/* ignore */

.				return BADTOKEN;

%%

#include <stdio.h>		/* for printf() */
#include <stdlib.h>		/* for atoi() */

main()
{
    extern char* yytext;	/* unless GNU flex is used, this line */
				/* should be "extern char yytext[];" */
    int token;

    while((token = yylex()) != END){
	printf("token no.%d: ", token);
        switch(token){
	case NUMBER :
	    printf("%d\n", atoi(yytext));
	    break;
	case RETURN :
	    printf("return\n");
	    break;
	default :
	    puts(yytext);
	    break;
	}
    }
}

